@@ -12,7 +12,14 @@ check_rpaframework_import()
 
 class SitesCore:
     def __init__(
-        self, login: str, password: str, url: str, timeout: int = 30, temp_folder: str = "", headless: bool = True
+        self,
+        login: str,
+        password: str,
+        auth_totp_key: str,
+        url: str,
+        timeout: int = 30,
+        temp_folder: str = "",
+        headless: bool = True,
     ):
         """
         Basic site core
@@ -20,12 +27,15 @@ class SitesCore:
         :param login: Login for the some sites
         :param password: Password for the some site
         :param url: URL for the some site
+        :param otp_code: OTP code for the some site
+        :param auth_totp_key: TOTP key for the some site
         :param timeout: Timeout for actions
         :param temp_folder: Path to temp folder (default download path)
         """
         self.url: str = url
         self.login: str = login
         self.password: str = password
+        self.auth_totp_key: str = auth_totp_key
         self.browser: Selenium = Selenium()
         self.browser.timeout = timeout
         self.temp_folder = temp_folder
@@ -33,6 +43,7 @@ class SitesCore:
 
         self.output_folder = os.path.join(os.environ.get("ROBOT_ROOT", os.getcwd()), "output")
         self.is_site_available: bool = False
+        self.is_password_expired: bool = False
         self.base_url: str = self.get_base_url(self.url)
 
     @staticmethod
