@@ -1,4 +1,5 @@
 import logging
+import requests
 import SeleniumLibrary.errors
 from ..basic.core import SitesCore
 from datetime import date, datetime, timedelta
@@ -6,6 +7,9 @@ from .exceptions import CentralReachException, ScheduledMaintenance
 import time
 import os
 import shutil
+import contextlib
+import pyotp
+from retry import retry
 
 
 class CentralReachCore(SitesCore):
@@ -13,12 +17,16 @@ class CentralReachCore(SitesCore):
         self,
         login: str,
         password: str,
+        auth_totp_key: str = "",
+        captcha_api_key: str | tuple[str, str] = "",
         url: str = "https://members.centralreach.com/#login",
         start_date: date = date.today().replace(day=1),
         end_date: date = date.today(),
         timeout: int = 30,
         temp_folder: str = "",
         headless: bool = False,
+        client_id: str = "",
+        client_secret: str = "",
     ):
         """
         CentralReachCore object. Please Inheritance it.
@@ -26,14 +34,108 @@ class CentralReachCore(SitesCore):
         :param login: login for CentralReach site.
         :param password: password for CentralReach site.
         :param url: url for CentralReach site.
+        :param otp_code: otp_code for CentralReach site.
+        :param auth_totp_key: auth_totp_key for CentralReach site.
+        :param captcha_api_key: api_key for captcha solver - can be just api_key itself or (provider, api_key) tuple.
         :param start_date: start_date for filters. The default value is first day of the month.
         :param end_date: end_date for filters. The default value is today.
         :param timeout: timeout for action. The default value is 30 seconds.
         :param temp_folder: Path to temp folder. The default value is temp/cr.
         """
-        super().__init__(login, password, url, timeout, temp_folder, headless)
+        super().__init__(login, password, auth_totp_key, url, timeout, temp_folder, headless)
         self.start_date: date = start_date
         self.end_date: date = end_date
+        self.client_id = client_id
+        self.client_secret = client_secret
+        self._do_auth = bool(self.client_id) and bool(self.client_secret)
+        if isinstance(captcha_api_key, str):
+            self.captcha_provider = "captcha.guru"
+            self.captcha_api_key = captcha_api_key
+        else:
+            self.captcha_provider = captcha_api_key[0]
+            self.captcha_api_key = captcha_api_key[1]
+
+    def _get_captcha_info(self):
+        """
+        Get all Google reCatpcha info. Obtains info for v2 and v3 captchas
+        """
+        data = self.browser.execute_javascript(
+            """
+        function findRecaptchaClients() {
+  // eslint-disable-next-line camelcase
+  if (typeof (___grecaptcha_cfg) !== 'undefined') {
+    // eslint-disable-next-line camelcase, no-undef
+    return Object.entries(___grecaptcha_cfg.clients).map(([cid, client]) => {
+      const data = { id: cid, version: cid >= 10000 ? 'V3' : 'V2' };
+      const objects = Object.entries(client).filter(([_, value]) => value && typeof value === 'object');
+
+      objects.forEach(([toplevelKey, toplevel]) => {
+        const found = Object.entries(toplevel).find(([_, value]) => (
+          value && typeof value === 'object' && 'sitekey' in value && 'size' in value
+        ));
+
+        if (typeof toplevel === 'object' && toplevel instanceof HTMLElement && toplevel['tagName'] === 'DIV'){
+            data.pageurl = toplevel.baseURI;
+        }
+
+        if (found) {
+          const [sublevelKey, sublevel] = found;
+
+          data.sitekey = sublevel.sitekey;
+          const callbackKey = data.version === 'V2' ? 'callback' : 'promise-callback';
+          const callback = sublevel[callbackKey];
+          if (!callback) {
+            data.callback = null;
+            data.function = null;
+          } else {
+            data.function = callback;
+            const keys = [cid, toplevelKey, sublevelKey, callbackKey].map((key) => `['${key}']`).join('');
+            data.callback = `___grecaptcha_cfg.clients${keys}`;
+          }
+        }
+      });
+      return data;
+    });
+  }
+  return [];
+}
+
+let res = findRecaptchaClients();
+return res
+        """
+        )
+        logging.info(data)
+        self.key = data[0]["sitekey"]
+        self.page_url = data[0]["pageurl"]
+        self.function = data[0]["function"]
+        self.callback = data[0]["callback"]
+
+    def _get_auth_token(self, client_id, client_secret):
+        """Retrieve the authentication token."""
+        data = {
+            "grant_type": "client_credentials",
+            "client_id": client_id,
+            "client_secret": client_secret,
+            "scope": "cr-api",
+        }
+
+        return requests.post("https://login.centralreach.com/connect/token", data=data)
+
+    def _get_internal_cookie(self, jwt):
+        """Exchange the authentication token for an internal session cookie."""
+        import requests
+
+        headers = {
+            "Accept": "application/json",
+        }
+
+        json_data = {
+            "token": jwt["access_token"],
+        }
+
+        return requests.post(
+            "https://members.centralreach.com/api/?framework.authtoken", headers=headers, json=json_data
+        )
 
     def login_to_site(self) -> bool:
         """
@@ -41,6 +143,10 @@ class CentralReachCore(SitesCore):
 
         :return: Status of the login. True if successfully log in
         """
+        username_field = '//*[@id="Username"]'
+        next_btn = '//*[@id="login"]'
+        password_field = '//*[@id="Password"]'
+
         self.is_site_available = False
         self.browser.close_browser()
 
@@ -53,17 +159,94 @@ class CentralReachCore(SitesCore):
         for i in range(1, 4):
             try:
                 self.browser.open_chrome_browser(self.url, headless=self._headless)
-                self.browser.set_window_size(1920, 1080)
-                self.browser.maximize_browser_window()
 
-                self.wait_element('//input[@type="password"]', is_need_screenshot=False, timeout=10)
+                # get API auth cookies for RPA automation
+                if self._do_auth:
+                    new_cookies = self._get_internal_cookie(
+                        self._get_auth_token(self.client_id, self.client_secret).json()
+                    )
+
+                    for cookie_name, cookie_value in new_cookies.cookies.get_dict().items():
+                        self.browser.add_cookie(str(cookie_name), str(cookie_value))
+
+                self.browser.set_window_size(1920, 1080)
+                self.wait_element(username_field, is_need_screenshot=False, timeout=10)
 
                 self.check_scheduled_maintenance()
 
-                self.wait_element('//input[@type="password"]')
-                self.browser.input_text_when_element_is_visible('//input[@type="text"]', self.login)
-                self.browser.input_text_when_element_is_visible('//input[@type="password"]', self.password)
-                self.browser.click_button_when_visible('//button[@id="login"]')
+                self.wait_element(username_field)
+                self.browser.input_text_when_element_is_visible(username_field, self.login)
+                self.browser.click_button_when_visible(next_btn)
+
+                self.wait_element(password_field)
+                self.browser.input_text_when_element_is_visible(password_field, self.password)
+
+                for attempt in range(1, 4):
+                    try:
+                        # Take screenshot before attempting captcha
+                        self.browser.capture_page_screenshot(
+                            os.path.join(self.output_folder, f"captcha_attempt_{attempt}_before.png")
+                        )
+                        logging.info(f"Starting captcha attempt {attempt}")
+                        logging.info("Getting captcha info")
+                        self._get_captcha_info()
+                        from twocaptcha import TwoCaptcha
+
+                        logging.info("Solving captcha with 2captcha")
+                        solver = TwoCaptcha(self.captcha_api_key)
+                        result = solver.recaptcha(
+                            sitekey=self.key, url=self.browser.get_location(), invisible=True, enterprise=1
+                        )
+                        code = result["code"]
+                        logging.info(f"Captcha code: {code}")
+                        logging.info("Injecting captcha code")
+                        self.browser.execute_javascript(
+                            "document.getElementById('g-recaptcha-response').innerHTML='{}'".format(code)
+                        )
+                        logging.info("Injecting captcha callback")
+                        self.browser.execute_javascript(self.callback + f"('{code}')")
+                        logging.info("Clicking next button")
+                        self.browser.click_button_when_visible(next_btn)
+                        validation_error = "//li[contains(text(), 'Recaptcha validation failed')]"
+                        if self.browser.does_page_contain_element(validation_error):
+                            logging.error("Captcha validation failed")
+                            self.browser.go_to("https://login.centralreach.com")
+                            raise Exception("Captcha validation failed")
+                        logging.info("Successfully Bypassed Captcha")
+
+                        # Take screenshot after successful captcha
+                        self.browser.capture_page_screenshot(
+                            os.path.join(self.output_folder, f"captcha_attempt_{attempt}_success.png")
+                        )
+                        break
+                    except KeyboardInterrupt:
+                        # Re-raise keyboard interrupt immediately
+                        raise
+                    except SystemExit:
+                        # Re-raise system exit immediately
+                        raise
+                    except BaseException as ex:
+                        # Catch all exceptions including those that don't inherit from Exception
+                        # Take screenshot after captcha failure
+                        self.browser.capture_page_screenshot(
+                            os.path.join(self.output_folder, f"captcha_attempt_{attempt}_failed.png")
+                        )
+
+                        logging.error(f"Failed to Bypass Captcha Attempt: {attempt}")
+                        logging.error(f"Exception type: {type(ex).__name__}")
+                        logging.error(f"Exception message: {str(ex)}")
+                        if attempt == 3:
+                            logging.info("Final captcha attempt failed, clicking button anyway")
+                            self.browser.click_button_when_visible(next_btn)
+                            break
+                        # Add a small delay between captcha attempts
+                        logging.info(f"Waiting 2 seconds before captcha attempt {attempt + 1}")
+                        time.sleep(2)
+                        continue
+
+                self.is_password_expired = self.check_password_expired()
+                if self.is_password_expired:
+                    break
 
                 if self.browser.does_page_contain_element(
                     '//div[@class="loginError"]/div[text()="There was an unexpected problem signing in. If you keep seeing this, please contact CentralReach Support"]'  # noqa
@@ -77,6 +260,7 @@ class CentralReachCore(SitesCore):
                             "Unexpected error during signing in. If you keep seeing this, contact CentralReach Support"
                         )
 
+                self.pass_mfa()
                 self.wait_element_and_refresh('//span[text()="Favorites"]', is_need_screenshot=False)
 
                 if self.does_element_displayed('//span[text()="Continue To Login"]'):
@@ -166,8 +350,8 @@ class CentralReachCore(SitesCore):
                     logging.warning(ex)
 
     def check_scheduled_maintenance(self):
-        if self.browser.does_page_contain_element("//div[text()='We’ll Be Back Soon!']"):
-            if self.browser.find_element("//div[text()='We’ll Be Back Soon!']").is_displayed():
+        if self.browser.does_page_contain_element("//div[contains(text(),'We’ll Be Back Soon')]"):
+            if self.browser.find_element("//div[contains(text(),'We’ll Be Back Soon')]").is_displayed():
                 self.browser.capture_page_screenshot(
                     os.path.join(self.output_folder, f'Maintenance_{datetime.now().strftime("%H_%M_%S")}.png')
                 )
@@ -333,3 +517,43 @@ class CentralReachCore(SitesCore):
             if valid_billing.lower() not in str(billing.text).lower():
                 self.change_billing("Billing", full_billing_name)
                 break
+
+    def check_password_expired(self):
+        with contextlib.suppress(AssertionError):
+            self.browser.wait_until_element_is_visible("//*[contains(text(), 'Your password has expired')]")
+            logging.info("Please reset AI worker password for Central Reach")
+            return True
+        return False
+
+    def reset_password(self, new_password):
+        self.wait_element('//input[@id="NewPassword"]')
+        self.browser.input_text_when_element_is_visible('//input[@id="NewPassword"]', self.login)
+
+        self.wait_element('//input[@id="ConfirmNewPassword"]')
+        self.browser.input_text_when_element_is_visible('//input[@id="ConfirmNewPassword"]', self.login)
+
+        self.wait_element('//button[@data-testid="update-password"]')
+        self.browser.click_element_when_visible('//button[@data-testid="update-password"]')
+
+        self.browser.close_all_browsers()
+
+    @retry(tries=3, delay=10)
+    def pass_mfa(self):
+        if not self.browser.is_element_visible('//*[@id="verify-code"]'):
+            return
+        if str(self.auth_totp_key):
+            try:
+                base32_key = self.auth_totp_key.replace(" ", "").upper()
+                totp = pyotp.TOTP(base32_key)
+                otp_code = totp.now()
+                self.wait_element('//*[@id="Code"]')
+                self.browser.input_text('//*[@id="Code"]', otp_code, clear=True)
+                self.browser.click_button_when_visible('//*[@id="verify-code"]')
+                time.sleep(5)
+                if self.browser.is_element_visible('//*[contains(text(),"Invalid code")]'):
+                    raise CentralReachException("Invalid MFA code")
+                self.browser.wait_until_element_is_not_visible('//*[@id="Code"]')
+            except Exception as ex:
+                raise CentralReachException(f"Error during MFA: {str(ex)}")
+        else:
+            raise CentralReachException("Auth Key is not provided")
