@@ -1,6 +1,4 @@
 from .core import QuickbooksCore
 
 
-__all__ = [
-    'QuickbooksCore'
-]
+__all__ = ["QuickbooksCore"]
